// +build aix darwin dragonfly freebsd linux netbsd openbsd solaris

package sipc_conn

import (
	"fmt"
	uuid "github.com/satori/go.uuid"
	"net"
	"os"
	"path"
)

type ServerCtx struct {
}

func GenerateServerName() string {
	return fmt.Sprintf("satoripc-%s.sock", uuid.Must(uuid.NewV4()).String())
}

func GeneratePath() string {
	return fmt.Sprintf("/var/run/%s", GenerateServerName())
}

func GenerateCustomPath(basePath4Unix string, customName string) string {
	return path.Join(basePath4Unix, customName)
}

func NewDefaultServer(name string) (*Server, error) {
	var sp string
	if name == "" {
		sp = GenerateServerName()
	} else {
		sp = name
	}
	return NewServer(sp, false)
}

func NewServer(path string, msgmode bool) (*Server, error) {
	o := &Server{
		serverPath:       path,
		isUnix:           true,
		connErrorHandler: defaultConnErrHandler,
		connHandler:      defaultConnHandler,
		ctx:              ServerCtx{},
	}
	return o, nil
}

func (s *Server) Win_UseSecurityDescrptor(sd string) {}
func (s *Server) Win_SetBufferSize(in, out int32)    {}

func (s *Server) Start() error {
	var err error
	os.Remove(s.serverPath)
	uaddr, err := net.ResolveUnixAddr("unix", s.serverPath)
	if err != nil {
		return err
	}
	s.listener, err = net.ListenUnix("unix", uaddr)
	if err != nil {
		return err
	}
	s.quitChan = make(chan int)
	go s.doAccept()
	return nil
}

func (s *Server) GetListenerAndListen() (net.Listener, error) {
	uaddr, err := net.ResolveUnixAddr("unix", s.serverPath)
	if err != nil {
		return nil, err
	}
	return net.ListenUnix("unix", uaddr)
}
